'use strict';
//System path library
const path = require('path');

//System file library
const fs = require('fs');

//Express library
const express = require('express');

//Body Parser library
const bodyParser = require('body-parser');

//CORS library
const cors = require('cors');

//Helmet library
const helmet = require('helmet');

//file upload
const multer = require('multer')

//API KEY UUID Generation
//const uuidAPIKey = require('uuid-apikey');

//Cassandra driver from DataStax
const cassandra = require('cassandra-driver');

//request tracker
const requestTracker = new cassandra.tracker.RequestLogger({ slowThreshold: 1000, logErroredRequests: true });
//
const { brotliDecompressSync } = require('zlib');
const CassandraClient = require('express-cassandra');
const { response } = require('express');
//const { level } = require('winston');
//const { response } = require('express');

//Importing all global declarations
const global = require(path.join(__dirname, './', 'middleware/globalDeclaration'));

//Express app
const mapp = express();

//Cassandra mapper query evalution library
const q = cassandra.mapping.q;

//Cassandra client initialization
const cclient = global.callFunction('connectDB', cassandra, global.KEY['HOST_LIST'], global.KEY['DATACENTER'], global.KEY['PORT'], global.KEY['KEYSPACE'], requestTracker,global.KEY['DBUSER'],global.KEY['DBPASSWORD']);
/*
cclient.on('log', (level, className, message, furtherInfo) => {
    console.log('log event: %s -- %s', level, message);
});
*/

//Cassandra mapper utility
const Mapper = cassandra.mapping.Mapper;

//Maplist of tables
const mapList = Object.getOwnPropertyNames(global.MAPPER).reduce((prev, key) => {
    prev[key] = { tables: [global.MAPPER[key]] };
    return prev;
}, {});

//Mapper initialization
const mapperObject = new Mapper(cclient, {
    models: mapList
});

//mapper, query options and others container object
const mapper = {
    'mapper': mapperObject,
    'q': q,
    'global': global,
    'cassandra': cassandra
}

//Query option definitions. Default values are set here. Change it if needed at required step.
const queryOptions = {
    queryHint: { prepare: true },
    cql: "SELECT now() FROM system.local;",
    executionProfile: "oltp",
    consistency: cassandra.types.consistencies.localOne
}

// create application/json parser
mapp.use(bodyParser.json())

// create application/x-www-form-urlencoded parser
mapp.use(bodyParser.urlencoded({ extended: true }))

//Definition of cors options for the app
const corsOptions = {
    origin: "*",
    methods: ["GET", "POST"],
    credentials: true,
    maxAge: 3600,
    optionsSuccessStatus: 200
}

//enable cors across app
mapp.use(cors(corsOptions));

//to protect from common well-kown vulnerability
mapp.use(helmet());

//log detail incoming request
mapp.use(function (req, res, next) {
    console.log(
        "\n Request's high level detail \n" +
        'Timestamp: ' + Date.now() + '\n' +
        'Base URL: ' + req.baseUrl + '\n' +
        'IP: ' + req.ip + '\n' +
        'Header: ' + JSON.stringify(req.headers) + '\n' +
        'URL: ' + req.url + '\n' +
        "#########"
    );
    next();
});

//function to handle error appwise
mapp.use(function (error, req, res, next) {
    console.error(error.stack)
    res.status(500).send('Something broke!')
});

requestTracker.emitter.on('slow', message => console.log(message));
requestTracker.emitter.on('failure', message => console.log(message));

/*********************************
 * Connectivity test  end points
 * 
 *********************************/

//test get end point
mapp.get('/', (req, res) => {
    res.json({
        'GET': [{
            'params': req.params,
            'body': req.body,
            'header': req.headers
        }]
    });
});

//test db connectivity post end point
mapp.post('/', (req, res) => {
    global.callFunction('testDB', cclient, mapper, queryOptions, req.params, req.body, req.headers).then((data) => {
        if (data instanceof Error) {
            console.log(data);
            res.json({ 'errorMsg': "DB connection failed." })
        }
        res.json({ 'successMsg': data })
    }).catch((err) => {
        console.log(err);
        res.json({ 'errorMsg': err })
    });
});

/***************************
 * Common end points
 * 
 ***************************/
/*
Function to get the active product list on page load.
*/
mapp.get('/getoffering/all', (req, res) => {
    global.callFunction('getAllProducts', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch products." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get the product list on page load.
*/
mapp.post('/getoffering/admin/all', (req, res) => {
    global.callFunction('getAllAdminProducts', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch products." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get the product using search term
*/
mapp.get('/getoffering/:search', (req, res) => {
    global.callFunction('getProductsBySearch', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch any product." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get the product list by name
*/
mapp.get('/getoffering/name/:name', (req, res) => {
    global.callFunction('getProductsByName', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch any product." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get the product list by category
*/
mapp.get('/getoffering/category/:category', (req, res) => {
    global.callFunction('getProductsByCategory', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch any product." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get the product list by casno
*/
mapp.get('/getoffering/casno/:casno', (req, res) => {
    global.callFunction('getProductsByCasno', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch any product." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get the product list by partner
*/
mapp.get('/getoffering/partner/:partner', (req, res) => {
    global.callFunction('getProductsByPartner', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch any product." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get the product list by country
*/
mapp.get('/getoffering/country/:country', (req, res) => {
    global.callFunction('getProductsByCountry', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch any product." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get the country list to pick in search option.
*/
mapp.get('/getcountrylist', (req, res) => {
    global.callFunction('getCountryList', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch country list." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get the currency list to pick in search option.
*/
mapp.get('/getcurrencylist', (req, res) => {
    global.callFunction('getCurrencyList', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch currency list." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get the currency exchange list to pick in search option.
*/
mapp.get('/xchange', (req, res) => {
    global.callFunction('getXchangeList', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch currency list." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});


/*
Function to get the gst map list to pick in search option.
*/
mapp.get('/gstmap', (req, res) => {
    global.callFunction('getGSTList', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch currency list." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get the partner list to pick in search option.
*/
mapp.get('/partnerlist', (req, res) => {
    global.callFunction('getPartnerList', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch partner list." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get the category list to pick in search option.
*/
mapp.get('/categorylist', (req, res) => {
    global.callFunction('getCategoryList', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch category list." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to submit communication
*/
mapp.post('/commany', (req, res) => {
    global.callFunction('postCommunication', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to upsert communication." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/***************************
 * User end points
 * 
 ***************************/

/*
Function to fetch account detail 
*/
mapp.post('/accountlist', (req, res) => {
    global.callFunction('getAccountList', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch any account." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

//get login list
mapp.post('/loginlist', (req, res) => {
    global.callFunction('getLoginList', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch any account." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to fetch contact detail 
*/
mapp.post('/contactlist', (req, res) => {
    global.callFunction('getContactList', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch any contact." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to fetch address detail 
*/
mapp.post('/addresslist', (req, res) => {
    global.callFunction('getAddressList', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch any address." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to fetch communication detail 
*/
mapp.post('/communication', (req, res) => {
    global.callFunction('getCommunicationList', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch any communication." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to fetch order detail 
*/
mapp.post('/orderlist', (req, res) => {
    global.callFunction('getOrderList', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch any account." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to fetch invoice detail 
*/
mapp.post('/invoicelist', (req, res) => {
    global.callFunction('getInvoiceList', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch any invoice." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to fetch service request detail 
*/
mapp.post('/srlist', (req, res) => {
    global.callFunction('getSRList', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch any service request." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to submit cart
*/
mapp.post('/addtocart', (req, res) => {
    global.callFunction('addToCart', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to upsert cart." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to submit acount
*/
mapp.post('/account', (req, res) => {
    let func = 'postAccount'
    if (req.body.System.action == 'update') {
        func = 'updateAccount'
    }
    global.callFunction(func, cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to upsert account." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to submit contact
*/
mapp.post('/contact', (req, res) => {
    global.callFunction('postContact', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to upsert contact." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to submit address
*/
mapp.post('/address', (req, res) => {
    global.callFunction('postAddress', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to upsert address." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});



/*
Function to get stat
*/
mapp.post('/stat', (req, res) => {
    global.callFunction('getStat', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to upsert communication." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to submit service request
*/
mapp.post('/sr', (req, res) => {
    global.callFunction('postSR', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to upsert service request." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to submit Orders
*/
mapp.post('/order', (req, res) => {
    global.callFunction('postOrder', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to upsert order." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});


/*
Function to submit user
*/
mapp.post('/user', (req, res) => {
    global.callFunction('postUser', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to upsert user." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});


/******************************
 * Login and Logout end points
 * 
 ******************************/

//Function for login
mapp.post('/login', (req, res) => {

    global.callFunction('login', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Login operationn failed." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

//Function for logout
mapp.post('/logout', (req, res) => {

    global.callFunction('logout', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong. Please clear browsing data and reopen." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});


/***************************
 * Admin end points
 * 
 ***************************/

/*
Function to submit products
*/
mapp.post('/product', (req, res) => {
    global.callFunction('postProductAdmin', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to upsert products." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to submit invoice
*/
mapp.post('/invoice', (req, res) => {
    global.callFunction('postInvoiceAdmin', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to upsert invoice." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/* Function to edit order item */
mapp.post('/orderitemAdmin', (req, res) => {
    global.callFunction('postOrderitemAdmin', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to update order item." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get system keys
*/
mapp.post('/getKeyAdmin', (req, res) => {
    global.callFunction('getKeyAdmin', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            } postXchangeAdmin
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get lov
*/
mapp.post('/getLovAdmin', (req, res) => {
    global.callFunction('getLovAdmin', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get GST
*/
mapp.post('/getGST', (req, res) => {
    global.callFunction('getGST', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get SR for admin
*/
mapp.post('/getSRListAdmin', (req, res) => {
    global.callFunction('getSRListAdmin', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get communication for admin
*/
mapp.post('/getCommunicationListAdmin', (req, res) => {
    global.callFunction('getCommunicationListAdmin', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get communication for admin
*/
mapp.post('/getInvoiceListAdmin', (req, res) => {
    global.callFunction('getInvoiceListAdmin', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to post ledger entry for admin
*/
mapp.post('/ledger', (req, res) => {
    global.callFunction('ledger', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get ledger entry for admin
*/
mapp.post('/getLedgerListAdmin', (req, res) => {
    global.callFunction('getLedgerListAdmin', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get company details
*/
mapp.post('/getCompanyAdmin', (req, res) => {
    global.callFunction('getCompanyAdmin', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});


/*
Function to fetch order detail for admin
*/
mapp.post('/orderlistadmin', (req, res) => {
    global.callFunction('getOrderListAdmin', cclient, mapper, queryOptions, req.params, req.body, req.headers)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Failed to fetch any account." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to update system keys
*/
mapp.post('/postKeyAdmin', (req, res) => {
    global.callFunction('postKeyAdmin', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.lCommonCommonog(err); res.json({ 'errorMsg': err }) });
});

/*
Function to update lov
*/
mapp.post('/postLovAdmin', (req, res) => {
    global.callFunction('postLovAdmin', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to update company details
*/
mapp.post('/postCompanyAdmin', (req, res) => {
    global.callFunction('postCompanyAdmin', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to update Exchange details
*/
mapp.post('/postXchangeAdmin', (req, res) => {
    global.callFunction('postXchangeAdmin', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to update Exchange details
*/
mapp.post('/postGSTAdmin', (req, res) => {
    global.callFunction('postGSTAdmin', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to upload bulk produts
*/
mapp.post('/bulkproducts', multer().any(), (req, res) => {
    global.callFunction('bulkproducts', cclient, mapper, queryOptions, req.params, req, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to upload bulk gst files
*/
mapp.post('/bulkgst', multer().any(), (req, res) => {
    global.callFunction('bulkgst', cclient, mapper, queryOptions, req.params, req, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});


/*
Function to upload bulk invoice files
*/
mapp.post('/bulkinv', multer().any(), (req, res) => {
    global.callFunction('bulkinv', cclient, mapper, queryOptions, req.params, req, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get order list
*/
mapp.post('/adminDataOrders', (req, res) => {
    global.callFunction('adminDataOrders', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get invoice list
*/
mapp.post('/adminDataInvoice', (req, res) => {
    global.callFunction('adminDataInvoice', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get ledger list
*/
mapp.post('/adminDataLedger', (req, res) => {
    global.callFunction('adminDataLedger', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get comm list
*/
mapp.post('/adminDataComm', (req, res) => {
    global.callFunction('adminDataComm', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to get SR list
*/
mapp.post('/adminDataSR', (req, res) => {
    global.callFunction('adminDataSR', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to send mail
*/
mapp.post('/sendemail', (req, res) => {
    global.callFunction('sendemail', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
        .then(data => {
            if (data instanceof Error) {
                console.log(data);
                res.json({ 'errorMsg': "Something went wrong." })
            } else {
                console.log(data)
                res.json({ 'successMsg': data })
            }
        })
        .catch(err => { console.log(err); res.json({ 'errorMsg': err }) });
});

/*
Function to download file
*/
mapp.post('/download', (req, res) => {

    if (req.body.context == 'INVOICE') {
        global.callFunction('downloadInvoice', cclient, mapper, queryOptions, req.params, req.body, req.headers, global.AUDIT)
            .then(data => {
                if (data instanceof Error) {
                    console.log(data);
                    res.send(null)
                } else {
                    console.log(data)
                    //res.setHeader('Content-Type', 'application/octet-stream')
                    res.download(data)
                }
            })
            .catch(err => { console.log(err); res.send(null) });
    } else {
        res.send(null);
    }

});

//Listen to connections
let port = 3000;
mapp.listen(port, () => {
    console.log(`Application is listining on port ${port}`);
});

/*
catch rejections and errors
*/
process.on('uncaughtException', function (error, origin) {
    console.log(`Exception occured ${error} from ${origin}`);
});

process.on('unhandledRejection', function (error, origin) {
    console.log(`Rejection occured ${error} from ${origin}`);
});